<?php
/**
 * apAuthentication
 * 
 * Abstract class to define the api the authenticators must implement
 * 
 * @property   array  $_authenticators   Static list of active authenticators
 * @property   string $_name             The friendly name of the authentication method
 * 
 * @method  registerAuthenticator()  static Registers and active authenticator
 * @method  array  getAuthenticators() static Returns the active authenticators
 * @method  string getName()  Returns the friendly name of the authentication method
 * @method  initialize()  Does what is necessary to be done upon initialization of the authenticator.  may identify the user if no user interaction is required
 * @method  process()  Processes the post and verifies the identity of the user
 * @method  render()  Renders the authenticator in html
 * 
 * @package    authpuppy
 * @subpackage model
 * @author     Geneviève Bastien <gbastien@versatic.net>
 * @version    bzr: $Id$
 */

abstract class apAuthentication {
  
  // List of active authenticators
  static $_authenticators = array();
  
  // Name of the authentication method
  protected $_name = "Generic authentication";
  // Name of the authentication sub type
  protected $_subtype = "";
  
  /**
   * Registers an active authenticator.  The class passed in parameter must
   *    be an instance of apAuthentication
   *    
   * @param $class  apAuthentication
   * @return 
   */
  public static function registerAuthenticator($class) {
    if (is_string($class)) {
      $authenticator = new $class();
    } elseif (is_object($class)) {
      $authenticator = $class;
    } else {
      throw new Exception ("apAuthenticator::registerAuthenticator, wrong parameter $class. expected string or object");
    }
    if ($authenticator instanceof apAuthentication) {
      self::$_authenticators[get_class($authenticator)] = $authenticator;
    } else {
      throw new Exception("$class not an authenticator");
    }
  }
  
  /**
   * Returns the list of authenticators
   * @return array of apAuthentication objects
   */
  public static function getAuthenticators() {
    return self::$_authenticators;
  }
  
  public function getName() {
    return $this->_name;
  }
  
  public function getSubType() {
    return $this->_subtype;
  }
  public function setSubType($subtype) {
    $this->_subtype = $subtype;
    return $this;
  }
  /**
   * Initializes the authenticator
   * This method can already identify the connection user if the method
   *   does not require user interaction
   * 
   * @param sfWebRequest $request the request
   * @param apBaseIdentity $identity the current identity
   * @return unknown_type
   */
  
  abstract public function initialize(sfWebRequest $request, apBaseIdentity $identity);
  
  /**
   * If the request was a post processes the content of the request 
   * The is where the authentication that requires user-interaction should take place
   * 
   * @param sfAction $action the Action calling this method
   * @param sfWebRequest $request the request
   * @param apBaseIdentity $identity the current identity
   * @return unknown_type
   */
  abstract public function process(sfAction $action, sfWebRequest $request, apBaseIdentity $identity);
  
  /**
   * Renders the authenticator in html.  This function is called in the context of a view
   * @return html string
   */
  abstract public function render();
  
}
